@@ -12,6 +12,12 @@ RSpec.configure do |c|
   # Readable test descriptions
   c.formatter = :documentation
 
+  # detect the situation where PUP-5016 is triggered and skip the idempotency tests in that case
+  # also note how fact('puppetversion') is not available because of PUP-4359
+  if fact('osfamily') == 'Debian' && fact('operatingsystemmajrelease') == '8' && shell('puppet --version').stdout =~ /^4\.2/
+    c.filter_run_excluding :skip_pup_5016 => true
+  end
+
   # Configure all nodes in nodeset
   c.before :suite do
     # Install module and dependencies
@@ -43,7 +49,18 @@ RSpec.configure do |c|
       else
         on host, puppet('module','install','puppetlabs-stdlib','--version','3.2.0')
         on host, puppet('module','install','stahnma/epel')
+        on host, puppet('module','install','puppet/staging')
       end
     end
   end
 end
+
+shared_examples "a idempotent resource" do
+  it 'should apply with no errors' do
+    apply_manifest(pp, :catch_failures => true)
+  end
+
+  it 'should apply a second time without changes', :skip_pup_5016 do
+    apply_manifest(pp, :catch_changes => true)
+  end
+end
